 // Disclaimer:
// This software is provided AS IS for educational purposes only,
// with no warranties, and confers no rights. Error checking is 
// elided for clarity. Carry out sufficient tests prior to using
// it in a productive environment.
// CASIO Europe GmbH accepts no responsibility for loss or damage
// arising from its use, including damage from virus.
//
// Please, note: This configuration information may be changed
// without any further notice.
// Casio Europe GmbH does not guarantee future compatibility. By 
// using this software you agree to these terms and conditions.
//
// Version 2.07 - May 11, 2012
// Supported Devices: IT-800, IT-3100, IT-9000, DT-X11, DT-X7, 
// DT-X8, and DT-X30
//
// Remarks: You need latest versions of patches and runtime
// libraries be installed onto your mobile device.
//////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using System.Globalization;
using Calib;    // -> CASIO Libraries

namespace ImagerDemo
{
    public partial class IMGSampleForm : Form
    {
        #region Fields

        private static int TriggerLeft;
        private static int TriggerRight;
        private static int TriggerCenter;
        private const int SystemLibrarySuccess = -1;
        private const int ImagerTimeout = 10000;
        // Increase the size if necessary:
        private const int BufferSize = 0xFF; 

        #endregion

        #region Trigger Key Handling

        /// <summary>
        /// Template for callback function.
        /// </summary>
        delegate int KeyStateDelegate();
        /// <summary>
        /// Callback function for IMGWaitForDecode.
        /// </summary>
        KeyStateDelegate keyStateDelegate = new KeyStateDelegate(OnKeyState);

        /// <summary>
        /// Called by IMGWaitForDecode in IMGSampleForm_KeyDown.
        /// </summary>
        /// <returns>The key state.</returns>
        /// <remarks>Change or add more keys here.</remarks>
        public static int OnKeyState()
        {
            if (NativeMethods.GetAsyncKeyState(TriggerLeft) < 0 ||
                NativeMethods.GetAsyncKeyState(TriggerRight) < 0 ||
                NativeMethods.GetAsyncKeyState(TriggerCenter) < 0)
            {
                return 1;   // Key is held down
            }
            else
            {
                return 0;   // Key is released
            }
        }

        #endregion

        #region IMGSampleForm

        /// <summary>
        /// Initializes a new instance of the <see cref="IMGSampleForm"/> class.
        /// </summary>
        public IMGSampleForm()
        {
            InitializeComponent();

            Resources.Culture = CultureInfo.CurrentCulture;
            Text = Resources.MainText;
            this.lblDataDescription.Text = Resources.LabelData;
            this.lblTypeDescription.Text = Resources.LabelType;
            this.lblHelp.Text = Resources.LabelHelp;
            this.btnExit.Text = Resources.ButtonExit;

            TriggerLeft = GetDefaultKey(SystemLibNet.Def.KEYID_LEFTTRIGGER);
            TriggerRight = GetDefaultKey(SystemLibNet.Def.KEYID_RIGHTTRIGGER);
            TriggerCenter = GetDefaultKey(SystemLibNet.Def.KEYID_CENTERTRIGGER);

            OpenImager();
        }

        /// <summary>
        /// Handles the Closing event of the IMGSampleForm control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void IMGSampleForm_Closing(object sender, CancelEventArgs e)
        {
            CloseImager();
        }

        /// <summary>
        /// Handles the Click event of the btnExit control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        #endregion

        #region Open Imager

        /// <summary>
        /// Opens the imager.
        /// </summary>
        /// <returns>Success of operation.</returns>
        private void OpenImager()
        {
            int result;

            result = IMGLibNet.Api.IMGInit();
            string path = GetIniFilePath();

            if (File.Exists(path) == true)
            {
                result = IMGLibNet.Api.IMGLoadConfigFile(path);
            }

            result = MakeParticularSettings();

            result = IMGLibNet.Api.IMGConnect();
        }

        #endregion

        #region Close Imager

        /// <summary>
        /// Closes the imager.
        /// </summary>
        private void CloseImager()
        {
            int result;

            result = IMGLibNet.Api.IMGDisconnect();
            result = IMGLibNet.Api.IMGDeinit();
        }

        #endregion

        #region Decoding

        /// <summary>
        /// Handles the KeyDown event of the IMGSampleForm control
        /// in order to activate the decoding.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
        private void IMGSampleForm_KeyDown(object sender, KeyEventArgs e)
        {
            if ((e.KeyValue == TriggerLeft ||
                e.KeyValue == TriggerRight ||
                e.KeyValue == TriggerCenter) &&
                e.KeyValue != -1)
            {
                int result;
                string pMessage = new string(' ', BufferSize);
                string pCodeID = new string(' ', 2);
                string pAimID = new string(' ', 3);
                string pSymModifier = new string(' ', 3);
                int pLength = 0;

                result = IMGLibNet.Api.IMGWaitForDecode(
                    ImagerTimeout,
                    pMessage,
                    pCodeID,
                    pAimID,
                    pSymModifier,
                    ref pLength,
                    Marshal.GetFunctionPointerForDelegate(keyStateDelegate));

                if (result == IMGLibNet.Def.IMG_SUCCESS && pLength > 0)
                {
                    this.lblData.Text = pMessage.Trim();
                    this.lblType.Text = GetType(pCodeID);
                }

                NativeMethods.MSG peekMsg = new NativeMethods.MSG();
                do
                {
                    if (NativeMethods.PeekMessage(
                        peekMsg,
                        IntPtr.Zero,
                        NativeMethods.WM_KEYFIRST,
                        NativeMethods.WM_KEYLAST,
                        NativeMethods.PM_REMOVE) == 0)
                    {
                        break;
                    }

                } while (true);
            }
        }

        #endregion

        #region Code Type

        /// <summary>
        /// Gets the type.
        /// </summary>
        /// <param name="pCodeID">The code ID.</param>
        /// <returns>Code type.</returns>
        private string GetType(string pCodeID)
        {
            string type;

            switch (pCodeID)
            {
                case "A ":
                    type = "Australian Postal";
                    break;
                case "z ":
                    type = "Aztec";
                    break;
                case "B ":
                    type = "British Postal";
                    break;
                case "C ":
                    type = "Canadian Postal";
                    break;
                case "a ":
                    type = "Codabar";
                    break;
                case "q ":
                    type = "Codablock F";
                    break;
                case "h ":
                    type = "Code 11";
                    break;
                case "j ":
                    type = "Code 128 or ISBT";
                    break;
                case "b ":
                    type = "Code 39";
                    break;
                case "l ":
                    type = "Code 49 or EAN 128";
                    break;
                case "i ":
                    type = "Code 93";
                    break;
                case "y ":
                    type = "RSS or UCC/EAN Composite";
                    break;
                case "w ":
                    type = "DataMatrix";
                    break;
                case "K ":
                    type = "Dutch Postal";
                    break;
                case "d ":
                    type = "EAN13";
                    break;
                case "D ":
                    type = "EAN8";
                    break;
                case "f ":
                    type = "IATA 2of5";
                    break;
                case "e ":
                    type = "ITF(Interleaved 2of5)";
                    break;
                case "J ":
                    type = "Japanese Postal";
                    break;
                case "x ":
                    type = "Maxicode";
                    break;
                case "R ":
                    type = "MicroPDF";
                    break;
                case "g ":
                    type = "MSI";
                    break;
                case "r ":
                    type = "PDF417";
                    break;
                case "L ":
                    type = "Planet Code";
                    break;
                case "P ":
                    type = "Postnet";
                    break;
                case "O ":
                    type = "OCR";
                    break;
                case "s ":
                    type = "QR";
                    break;
                case "T ":
                    type = "TLC39";
                    break;
                case "c ":
                    type = "UPC Version A";
                    break;
                case "E ":
                    type = "UPC Version E0,E1";
                    break;
                default:
                    type = "Unknown Symbol";
                    break;
            }

            return type;
        }

        #endregion

        #region Settings

        /// <summary>
        /// Makes particular settings.
        /// </summary>
        /// <returns>Success of operation as a return code from IMGLibNet.</returns>
        /// <remarks>In this sample EAN8 is just enabled,
        /// while all other settings for that symbology are conserved.</remarks>
        private int MakeParticularSettings()
        {
            int result;
            bool bEnabled = false;
            bool bXmitCheckChar = false;
            bool bAddendaReq = false;
            bool bAddendaSeperator = false;
            bool bAddenda2Digit = false;
            bool bAddendenda5Digit = false;

            result = IMGLibNet.Api.IMGGetEAN13(
                ref bEnabled,
                ref bXmitCheckChar,
                ref bAddendaReq,
                ref bAddendaSeperator,
                ref bAddenda2Digit,
                ref bAddendenda5Digit);

            if (result == IMGLibNet.Def.IMG_SUCCESS)
            {
                result = IMGLibNet.Api.IMGSetEAN13(
                    true,
                    bXmitCheckChar,
                    bAddendaReq,
                    bAddendaSeperator,
                    bAddenda2Digit,
                    bAddendenda5Digit);
            }

            return result;
        } 

        #endregion

        #region System Library

        /// <summary>
        /// Gets the key value for a given key.
        /// </summary>
        /// <returns>The key value if successful; otherwise -1.</returns>
        private int GetDefaultKey(int keyId)
        {
            int defaultKey = -1;
            int[] buffer = new int[16];

            if (SystemLibNet.Api.SysGetDefaultKey(keyId, buffer) == SystemLibrarySuccess)
            {
                defaultKey = buffer[0] & 0xFF;
            }

            return defaultKey;
        }

        /// <summary>
        /// Gets the ini file path.
        /// </summary>
        /// <returns>The full path to the ini file.</returns>
        private string GetIniFilePath()
        {
            int result;
            char[] buffer = new char[256];
            string path = "";

            result = SystemLibNet.Api.SysGetSystemName(
                SystemLibNet.Def.IMGIni_FilePath, buffer);

            if (result == SystemLibrarySuccess)
            {              
                path = new string(buffer);
                path = path.TrimEnd('\0');
            }

            return path;
        }
 
	    #endregion

        #region NativeMethods

        internal sealed class NativeMethods
        {

            private NativeMethods()
            {
            }

            internal const int WM_KEYFIRST = 0x0100;
            internal const int WM_KEYLAST = 0x0108;
            internal const int PM_REMOVE = 0x0001;

            internal class MSG
            {
                private IntPtr hwnd;
                private int message;
                private IntPtr wParam;
                private IntPtr lParam;
                private int time;
                [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2)]
                private int[] pt;

                internal MSG()
                {
                    this.hwnd = IntPtr.Zero;
                    this.message = default(int);
                    this.wParam = IntPtr.Zero;
                    this.lParam = IntPtr.Zero;
                    this.time = default(int);
                    this.pt = new int[2];
                }

                internal int Message
                {
                    get
                    {
                        return this.message;
                    }
                }

                internal int Time
                {
                    get
                    {
                        return this.time;
                    }
                }
            }

            [DllImport("coredll.dll", SetLastError = false)]
            internal static extern short GetAsyncKeyState(int vKey);

            [DllImport("coredll.dll", SetLastError = false)]
            internal static extern int PeekMessage(
                MSG lpMsg,
                IntPtr hWnd,
                int wMsgFilterMin,
                int wMsgFilterMax,
                int wRemoveMsg);
        }

         #endregion
    }
}